package kcg.faics.susa.service.impl;

import java.util.List;

import kcg.faics.susa.vo.SusaFileVO;

import org.springframework.stereotype.Repository;

import egovframework.rte.psl.dataaccess.EgovAbstractMapper;

/**
 * 외사수사 관련 파일 Mapper 클래스.
 * 
 * @author kimnomin
 *
 */
@Repository("susaFileMapper")
public class SusaFileMapper extends EgovAbstractMapper {
	
	/**
	 * 파일 정보를 반환한다.
	 * 
	 * @param fileVO 파일 VO
	 * @return 파일 정보
	 */
	public SusaFileVO selectFile(final SusaFileVO fileVO) {
		return selectOne("SusaFile.select", fileVO);
	}
	
	/**
	 * 파일 목록을 반환한다.
	 * 
	 * @param fileVO 파일 VO 
	 * @return 파일 목록
	 */
	public List<SusaFileVO> selectFiles(final SusaFileVO fileVO) {
		return selectList("SusaFile.selectList", fileVO);
	}

	/**
	 * 파일 정보를 입력한다.
	 * 
	 * @param fileVO 파일 VO
	 * @return 1-성공, 0-실패
	 */
	public int insertFile(final SusaFileVO fileVO) {
		return insert("SusaFile.insert", fileVO);
	}
	
	/**
	 * 파일 정보를 삭제한다.
	 * 
	 * @param fileVO 파일 VO
	 * @return 1-성공, 0-실패
	 */
	public int deleteFile(final SusaFileVO fileVO) {
		return delete("SusaFile.delete", fileVO);
	}
}
